﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

using AIOUSBNet;  // the namespace exposes the AIOUSB Class interface 

/* ADC Continuous Callback acquisition Sample
 * This sample acquires using a callback function (myCallBack) which stores the counts received into a global array ... but overwrites most of the data
 * saving only the last scan worth.
 * The sample displays data in a timer event handler at 10Hz, simply showing the most recent scan of data as acquired by the Callback.
 * 
 * In a real-world application the callback wouldn't overwrite the ChannelCounts[] values, it might log or FFT or whatnot instead.
 * 
 * Look for "change this!" comments.
 */


namespace Sample1
{
    
    public partial class Form1 : Form
    {
        public static AIOUSB.ADCallback myCallBack;
        

        public const int ERROR_SUCCESS = 0; // Win32 standard response code
        private const byte bmDifferential = 0x8; // const for Range Selection to indicate "Differential"
        private UInt32 ConfigBufSize = 20; // sizeof(Config);

        public UInt32 DeviceIndex;        
        public byte[] ConfigBuf = new byte[20];
        
        UInt16[] ChannelCounts = new UInt16[8];        // buffer for final UInt16 count data
        private int counter;

        public Form1()
        {
            InitializeComponent();
            myCallBack = new AIOUSB.ADCallback(ADCallbackReport);
            counter = 0;
        }
        
        private void Form1_Load(object sender, EventArgs e)
        {
            DeviceIndex = AIOUSB.diOnly; // Initialize default Device Only:
        }

        private void ExecuteButton_Click(object sender, EventArgs e)
        {
            DisplayTimer.Stop();

            AIOUSB.AIOUSB_ReloadDeviceLinks();

            DeviceIndex = AIOUSB.ResolveDeviceIndex(AIOUSB.diOnly);
            if (DeviceIndex == AIOUSB.diNone)
            {
                // handle error
                return;
            }
            
            byte ADCRangeCode = 1; // ±10V input range: change here!

            for (int i = 0; i < 16; i++)
            {
                ConfigBuf[i] = (byte)(bmDifferential | ADCRangeCode); 
            }
            ConfigBuf[16] = 0; // not calibrate mode
            ConfigBuf[17] = 5; // timer scan start
            ConfigBuf[18] = 0x70; // channel 0 through 7
            ConfigBuf[19] = 0; // no oversamples

            var status = AIOUSB.ADC_SetConfig(DeviceIndex, ConfigBuf, ref ConfigBufSize);
            if (status != ERROR_SUCCESS)
            {
                // handle error
                return;
            }

            double Hz = 0.0;
            AIOUSB.CTR_StartOutputFreq(DeviceIndex, 0, ref Hz);
            AIOUSB.ADC_BulkContinuousCallbackStart(DeviceIndex, 256*2*8, 32, 0, myCallBack);
            Hz = 2000.0; // 2000 Hz scan starts: change here!
            AIOUSB.CTR_StartOutputFreq(DeviceIndex, 0, ref Hz);

            ExecuteButton.Enabled = false;
            StopButton.Enabled = true;

            DisplayTimer.Start();
        }


        public void ADCallbackReport(IntPtr pBuf, UInt32 BufSize, UInt32 Flags, UInt32 Context)
        {
            if (((Flags & 2) != 0)) // This flag (2) tells us we've got the last continuous callback packet:
            {
                // in case we care               
            }

            int BufLen = ((int)BufSize / 2); // length of the buffer in 16 bit elements instead of 8-bit
            Int16[] Buf = new Int16[BufLen]; // array for Int16 copy of buf for marshalling

            // Copy pBuf into an Int16 array then convert to UInt16 when used: 
            // This is required due to CLR restraints on IntPtr and Marshal.Copy() for unmanaged code array pointers
            if (BufLen != 0) // if there is data to copy
            {
                Marshal.Copy(pBuf, Buf, 0, (Buf.Length)); // copy
                int NextChannel = 0;
                for (int i = 0; i < BufLen; i++)
                {
                    ChannelCounts[NextChannel] = (UInt16)Buf[i];
                    NextChannel = (NextChannel + 1) % 8;
                }
                counter++;
            }
        }

        private void DisplayTimer_Tick(object sender, EventArgs e)
        {
            listBoxVoltage.Items.Clear(); 
            for (int I = 0; I < 8; ++I)
            {
                double Volts = ChannelCounts[I] / 65536.0 * 20.0 - 10.0; // scale for ±10V range (rangecode 1): change here!
                string strChVolt = String.Format("Ch {0,2:D2}: {1,8:F4}", I, Volts);
                listBoxVoltage.Items.Add(strChVolt);
            }
            listBoxVoltage.Items.Add("total callbacks: "+counter.ToString());
        }

        private void StopButton_Click(object sender, EventArgs e)
        {
            DisplayTimer.Stop();
            AIOUSB.ADC_BulkContinuousEnd(DeviceIndex, 0);

            ExecuteButton.Enabled = true;
            StopButton.Enabled = false;
        }
    }
}
